package backend

import (
	"context"
	"fmt"
	"kpt-pasture/model"
	"net/http"
	"strconv"
	"strings"

	pasturePb "gitee.com/xuyiping_admin/go_proto/proto/go/backend/cow"
	"gitee.com/xuyiping_admin/pkg/xerr"
	"gorm.io/gorm"
)

func (s *StoreEntry) EventTemplate(ctx context.Context, req *pasturePb.SearchEventRequest) {

}

func (s *StoreEntry) CreateEventTemplate(ctx context.Context, req *pasturePb.SearchEventRequest) *pasturePb.EventData {
	return nil
}

func (s *StoreEntry) ParseCowIds(ctx context.Context, cowId string) ([]*model.Cow, error) {
	if len(cowId) == 0 {
		return nil, xerr.Custom("cow id is required")
	}
	cowIdStr := strings.Split(cowId, ",")
	var cowIds = make([]*model.Cow, 0)
	for _, v := range cowIdStr {
		cowId, err := strconv.ParseInt(v, 10, 64)
		cow, err := s.GetCowInfoByCowId(ctx, cowId)
		if err != nil {
			return nil, xerr.WithStack(err)
		}
		cowIds = append(cowIds, cow)
	}

	return cowIds, nil
}

func (s *StoreEntry) EnterList(ctx context.Context, req *pasturePb.SearchEventRequest, pagination *pasturePb.PaginationModel) (*pasturePb.SearchEnterEventResponse, error) {
	eventEnterList := make([]*model.EventEnter, 0)
	var count int64 = 0
	pref := s.DB.Model(new(model.EventEnter))
	if len(req.CowId) > 0 {
		cowIds := strings.Split(req.CowId, ",")
		pref.Where("cow_id IN ?", cowIds)
	}

	if err := pref.Order("id desc").
		Count(&count).Limit(int(pagination.PageSize)).
		Offset(int(pagination.PageOffset)).
		Find(&eventEnterList).Error; err != nil {
		return nil, xerr.WithStack(err)
	}

	penMap := s.PenMap(ctx)
	breedStatusMap := s.CowBreedStatusMap()
	cowSourceMap := s.CowSourceMap()
	cowTypeMap := s.CowTypeMap()
	cowKindMap := s.CowKindMap()

	return &pasturePb.SearchEnterEventResponse{
		Code:    http.StatusOK,
		Message: "ok",
		Data: &pasturePb.SearchEnterEventData{
			List:     model.EventEnterSlice(eventEnterList).ToPB(penMap, breedStatusMap, cowSourceMap, cowTypeMap, cowKindMap),
			Total:    int32(count),
			PageSize: pagination.PageSize,
			Page:     pagination.Page,
		},
	}, nil
}

func (s *StoreEntry) CreateEnter(ctx context.Context, req *pasturePb.EventEnterRequest) error {
	newCowData := model.NewCow(req)
	currentUser, err := s.GetCurrentSystemUser(ctx)
	if err != nil {
		return xerr.Customf("当前用户信息错误")
	}

	req.MessengerName = currentUser.Name
	if req.MessengerId > 0 {
		systemUser, _ := s.GetSystemUserById(ctx, int64(req.MessengerId))
		req.OperationName = systemUser.Name
	}

	newEventEnter := model.NewEventEnter(newCowData.Id, req)
	if err = s.DB.Transaction(func(tx *gorm.DB) error {
		if err = tx.Create(newCowData).Error; err != nil {
			return xerr.WithStack(err)
		}
		if err = tx.Create(newEventEnter).Error; err != nil {
			return xerr.WithStack(err)
		}
		return nil
	}); err != nil {
		return xerr.WithStack(err)
	}
	return nil
}

func (s *StoreEntry) GroupTransferList(ctx context.Context, req *pasturePb.SearchEventRequest, pagination *pasturePb.PaginationModel) (*pasturePb.SearchTransferGroupEventResponse, error) {
	eventGroupTransferList := make([]*pasturePb.EventTransferGroupData, 0)
	var count int64 = 0
	pref := s.DB.Table(fmt.Sprintf("%s as a", new(model.EventTransferGroup).TableName())).
		Select(`a.id,a.cow_id,a.transfer_in_pen_id,a.transfer_out_pen_id,a.lact,a.remarks,a.transfer_reason_id,
			a.transfer_date,a.created_at,a.staff_member_id,b.name as transfer_in_pen_name,c.name as transfer_out_pen_name,
			d.name as transfer_reason_name,e.name as staff_member_name,f.lact,f.ear_number`).
		Joins(fmt.Sprintf("JOIN %s AS b ON a.transfer_in_pen_id = b.id", new(model.Pen).TableName())).
		Joins(fmt.Sprintf("JOIN %s AS c on a.transfer_out_pen_id = c.id", new(model.Pen).TableName())).
		Joins(fmt.Sprintf("JOIN %s AS d on a.transfer_reason_id = d.id", new(model.ConfigTransferPenReason).TableName())).
		Joins(fmt.Sprintf("JOIN %s AS e ON a.staff_member_id = e.id", new(model.SystemUser).TableName())).
		Joins(fmt.Sprintf("JOIN %s AS f ON a.cow_id = f.id", new(model.Cow).TableName()))
	if len(req.CowId) > 0 {
		cowIds := strings.Split(req.CowId, ",")
		pref.Where("a.cow_id IN ?", cowIds)
	}

	if err := pref.Order("a.id desc").Group("a.id").
		Count(&count).Limit(int(pagination.PageSize)).
		Offset(int(pagination.PageOffset)).
		Find(&eventGroupTransferList).Error; err != nil {
		return nil, xerr.WithStack(err)
	}

	return &pasturePb.SearchTransferGroupEventResponse{
		Code:    http.StatusOK,
		Message: "ok",
		Data: &pasturePb.SearchTransferGroupEventData{
			List:     eventGroupTransferList,
			Total:    int32(count),
			PageSize: pagination.PageSize,
			Page:     pagination.Page,
		},
	}, nil
}

func (s *StoreEntry) CreateGroupTransfer(ctx context.Context, req *pasturePb.TransferGroupEventRequest) error {
	system, _ := s.GetCurrentSystemUser(ctx)
	res := make([]*model.EventTransferGroup, 0)
	for _, v := range req.Body {
		cow, err := s.GetCowInfoByCowId(ctx, int64(v.CowId))
		if err != nil {
			return xerr.WithStack(err)
		}
		// 转去栏舍和当前栏舍相同，则不处理
		if cow.PenId == v.TransferInPenId {
			continue
		}
		pen, err := s.GetPenById(ctx, int64(v.TransferInPenId))
		if err != nil {
			return xerr.WithStack(err)
		}

		transferReasonData, err := s.GetTransferReasonInfo(ctx, int64(v.TransferReasonId))
		if err != nil {
			return xerr.WithStack(err)
		}

		staffMemberData, err := s.GetSystemUserById(ctx, int64(v.StaffMemberId))
		if err != nil {
			return xerr.WithStack(err)
		}

		res = append(res, &model.EventTransferGroup{
			CowId:            int64(v.CowId),
			TransferInPenId:  int64(v.TransferInPenId),
			TransferOutPenId: pen.Id,
			Lact:             cow.Lact,
			DayAge:           cow.GetDayAge(),
			TransferDate:     v.TransferDate,
			TransferReasonId: transferReasonData.Id,
			Remarks:          v.Remarks,
			StaffMemberId:    staffMemberData.Id,
			OperationId:      system.Id,
		})
	}
	if len(res) <= 0 {
		return nil
	}

	if err := s.DB.Transaction(func(tx *gorm.DB) error {
		if err := tx.Create(res).Error; err != nil {
			return xerr.WithStack(err)
		}
		for _, v := range res {
			cow, err := s.GetCowInfoByCowId(ctx, v.CowId)
			if err != nil {
				return xerr.WithStack(err)
			}
			if err = s.DB.Model(cow).Update("pen_id", v.TransferInPenId).Error; err != nil {
				return xerr.WithStack(err)
			}
		}
		return nil
	}); err != nil {
		return xerr.WithStack(err)
	}

	return nil
}

func (s *StoreEntry) BodyScoreList(ctx context.Context, req *pasturePb.SearchEventRequest, pagination *pasturePb.PaginationModel) (*pasturePb.SearchBodyScoreEventResponse, error) {
	bodyScoreList := make([]*pasturePb.BodyScoreList, 0)
	var count int64 = 0
	pref := s.DB.Table(fmt.Sprintf("%s as a", new(model.EventBodyScore).TableName())).
		Select(`a.id,a.cow_id,a.ear_number,a.score as body_score,a.lact,a.day_age,a.score_at,a.remarks,a.created_at,
		a.updated_at,a.staff_member_id,a.operation_id,b.name as staff_member_name,c.name as operation_name`).
		Joins(fmt.Sprintf("JOIN %s AS b on a.staff_member_id = b.id", new(model.SystemUser).TableName())).
		Joins(fmt.Sprintf("JOIN %s AS c on a.operation_id = c.id", new(model.SystemUser).TableName()))
	if len(req.CowId) > 0 {
		cowIds := strings.Split(req.CowId, ",")
		pref.Where("a.cow_id IN ?", cowIds)
	}

	if err := pref.Order("a.id desc").
		Count(&count).Limit(int(pagination.PageSize)).
		Offset(int(pagination.PageOffset)).
		Find(&bodyScoreList).Error; err != nil {
		return nil, xerr.WithStack(err)
	}

	return &pasturePb.SearchBodyScoreEventResponse{
		Code:    http.StatusOK,
		Message: "ok",
		Data: &pasturePb.SearchBodyScoreData{
			List:     bodyScoreList,
			Total:    int32(count),
			PageSize: pagination.PageSize,
			Page:     pagination.Page,
		},
	}, nil
}

func (s *StoreEntry) CreateBodyScore(ctx context.Context, req *pasturePb.BodyScoreEventRequest) error {
	if len(req.CowId) <= 0 {
		return xerr.Custom("请选择相关牛只")
	}
	currentSystemUser, _ := s.GetCurrentSystemUser(ctx)

	//cowList := strings.Split(req.CowId, ",")
	bodyScourEvent := make([]*model.EventBodyScore, 0)
	cowList, err := s.ParseCowIds(ctx, req.CowId)
	if err != nil {
		return xerr.WithStack(err)
	}
	for _, cow := range cowList {
		bodyScourEvent = append(bodyScourEvent, model.NewEventBodyScore(cow, currentSystemUser.Id, req))
	}
	if len(bodyScourEvent) <= 0 {
		return nil
	}

	return s.DB.Create(bodyScourEvent).Error
}

func (s *StoreEntry) WeightList(ctx context.Context, req *pasturePb.SearchEventRequest, pagination *pasturePb.PaginationModel) (*pasturePb.SearchWeightEventResponse, error) {
	weightList := make([]*pasturePb.SearchWeightList, 0)
	var count int64 = 0
	pref := s.DB.Table(fmt.Sprintf("%s as a", new(model.EventWeight).TableName())).
		Select(`a.id,a.cow_id,a.ear_number,a.weight / 100 as weight,a.lact,a.day_age,a.weight_at,a.remarks,a.created_at,
		a.updated_at,a.staff_member_id,a.operation_id,b.name as staff_member_name,c.name as operation_name`).
		Joins(fmt.Sprintf("JOIN %s AS b on a.staff_member_id = b.id", new(model.SystemUser).TableName())).
		Joins(fmt.Sprintf("JOIN %s AS c on a.operation_id = c.id", new(model.SystemUser).TableName()))
	if len(req.CowId) > 0 {
		cowIds := strings.Split(req.CowId, ",")
		pref.Where("a.cow_id IN ?", cowIds)
	}

	if err := pref.Order("a.id desc").
		Count(&count).Limit(int(pagination.PageSize)).
		Offset(int(pagination.PageOffset)).
		Find(&weightList).Error; err != nil {
		return nil, xerr.WithStack(err)
	}

	return &pasturePb.SearchWeightEventResponse{
		Code:    http.StatusOK,
		Message: "ok",
		Data: &pasturePb.SearchWeightData{
			List:     weightList,
			Total:    int32(count),
			PageSize: pagination.PageSize,
			Page:     pagination.Page,
		},
	}, nil
}

func (s *StoreEntry) WeightCreate(ctx context.Context, req *pasturePb.EventWeight) error {
	if len(req.CowId) <= 0 {
		return xerr.Custom("请选择相关牛只")
	}
	currentSystemUser, _ := s.GetCurrentSystemUser(ctx)
	weightEvent := make([]*model.EventWeight, 0)
	for _, item := range req.WeightItems {
		cow, err := s.GetCowInfoByCowId(ctx, int64(item.CowId))
		if err != nil {
			return xerr.WithStack(err)
		}
		var weight = int32(item.Weight * 100)
		weightEvent = append(weightEvent, model.NewEventWeight(cow, currentSystemUser, weight, req))
	}
	if len(weightEvent) <= 0 {
		return nil
	}

	if err := s.DB.Transaction(func(tx *gorm.DB) error {
		for _, item := range weightEvent {
			if err := tx.Model(new(model.Cow)).Where("id = ?", item.CowId).Updates(map[string]interface{}{
				"last_weight_at": item.WeightAt,
				"current_weight": item.Weight,
			}).Error; err != nil {
				return xerr.WithStack(err)
			}
		}
		if err := tx.Create(weightEvent).Error; err != nil {
			return xerr.WithStack(err)
		}
		return nil
	}); err != nil {
		return xerr.WithStack(err)
	}
	return nil
}
